
/* Chrysalide - Outil d'analyse de fichiers binaires
 * relative.c - opérandes d'adresses relatives
 *
 * Copyright (C) 2012-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "relative.h"


#include "../../operand-int.h"



/* Définition d'un opérande x86 d'adresse relative (instance) */
struct _GX86RelativeOperand
{
    GArchOperand parent;                    /* Instance parente            */

    GImmOperand *immediate;                 /* Adresse visée reconstituée  */

};

/* Définition d'un opérande x86 d'adresse relative (classe) */
struct _GX86RelativeOperandClass
{
    GArchOperandClass parent;               /* Classe parente              */

};


/* Initialise la classe des opérandes x86 d'adresse relative. */
static void g_x86_relative_operand_class_init(GX86RelativeOperandClass *);

/* Initialise une instance d'opérande x86 d'adresse relative. */
static void g_x86_relative_operand_init(GX86RelativeOperand *);



/* Indique le type défini par la GLib pour un opérande x86 d'adresse relative. */
G_DEFINE_TYPE(GX86RelativeOperand, g_x86_relative_operand, G_TYPE_ARCH_OPERAND);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des opérandes x86 d'adresse relative.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_relative_operand_class_init(GX86RelativeOperandClass *klass)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise une instance d'opérande x86 d'adresse relative.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_relative_operand_init(GX86RelativeOperand *operand)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : data = flux de données à analyser.                           *
*                pos  = position courante dans ce flux. [OUT]                 *
*                len  = taille totale des données à analyser.                 *
*                size = taille de l'opérande, et donc du registre.            *
*                base = adresse de référence pour le calcul.                  *
*                                                                             *
*  Description : Crée un opérande X86 d'adresse relative.                     *
*                                                                             *
*  Retour      : Opérande mis en place.                                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_x86_relative_operand_new(const bin_t *data, off_t *pos, off_t len, MemoryDataSize size, vmpa_t base)
{
    GX86RelativeOperand *result;            /* Structure à retourner       */
    off_t init_pos;                         /* Position avant lecture      */
    int8_t val8;                            /* Valeur sur 8 bits           */
    int16_t val16;                          /* Valeur sur 16 bits          */
    int32_t val32;                          /* Valeur sur 32 bits          */
    vmpa_t address;                         /* Adresse finale visée        */

    init_pos = *pos;

    switch (size)
    {
        case MDS_8_BITS_UNSIGNED:
            read_s8(&val8, data, pos, len, SRE_LITTLE /* FIXME */);
            address = base + (*pos - init_pos) + val8;
            break;
        case MDS_16_BITS_UNSIGNED:
            read_s16(&val16, data, pos, len, SRE_LITTLE /* FIXME */);
            address = base + (*pos - init_pos) + val16;
            break;
        case MDS_32_BITS_UNSIGNED:
            read_s32(&val32, data, pos, len, SRE_LITTLE /* FIXME */);
            address = base + (*pos - init_pos) + val32;
            break;
        default:
            return NULL;
            break;
    }

    result = g_object_new(G_TYPE_X86_RELATIVE_OPERAND, NULL);
    result->immediate = g_imm_operand_new_from_value(MDS_32_BITS/*FIXME*/, (uint32_t)address/* FIXME */);

    return G_ARCH_OPERAND(result);

}


#if 0
/******************************************************************************
*                                                                             *
*  Paramètres  : operand   = opérande à transcrire.                           *
*                options   = options de rendu.                                *
*                rendering = support effectif final des lignes de code.       *
*                stream    = flux ouvert en écriture.                         *
*                                                                             *
*  Description : Ajoute du texte simple à un fichier ouvert en écriture.      *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_relative_operand_add_text(const GX86RelativeOperand *operand, GRenderingOptions *options, MainRendering rendering, FILE *stream)
{
    g_content_exporter_add_text(G_CONTENT_EXPORTER(operand->immediate), options, rendering, stream);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à transcrire.                             *
*                buffer  = espace où placer ledit contenu.                    *
*                options = options de rendu.                                  *
*                                                                             *
*  Description : Ajoute à un tampon GLib le contenu de l'instance spécifiée.  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_x86_relative_operand_to_buffer(const GX86RelativeOperand *operand, GBufferLine *buffer, GRenderingOptions *options)
{
    g_content_exporter_to_buffer(G_CONTENT_EXPORTER(operand->immediate), buffer, options);

}
#endif


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                                                                             *
*  Description : Fournit l'adresse représentée par une opérande X86.          *
*                                                                             *
*  Retour      : Valeur portée par l'opérande.                                *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

const GImmOperand *g_x86_relative_operand_get_value(const GX86RelativeOperand *operand)
{
    return operand->immediate;

}
